@@ -94,6 +94,10 @@ gnome_keyring_free (GnomeKeyring *keyring)
 	GList *l;
 	GnomeKeyringItem *item;
 
+	if (keyring == NULL) {
+		return;
+	}
+
 	l = keyring->items;
 	while (l != NULL) {
 		item = l->data;
@@ -297,7 +301,10 @@ hash_int (guint32 x)
 static char *
 hash_string (const char *str)
 {
-        guchar digest[16];
+	guchar digest[16];
+
+	if (str == NULL)
+		return NULL;
 	
 	gnome_keyring_md5_string (str, digest);
 	return gnome_keyring_md5_digest_to_ascii (digest);
@@ -550,6 +557,10 @@ match_attributes (GnomeKeyringItem *item,
 				}
 				switch (attribute->type) {
 				case GNOME_KEYRING_ATTRIBUTE_TYPE_STRING:
+					if ((attribute->value.string == NULL || item_attribute->value.string == NULL) && 
+					    attribute->value.string != item_attribute->value.string) {
+						return FALSE;
+					}
 					if (strcmp (attribute->value.string, item_attribute->value.string) != 0) {
 						return FALSE;
 					}
@@ -2985,9 +2996,17 @@ main (int argc, char *argv[])
 		
 	}
 	
+#ifdef WITH_DBUS
+	gnome_keyring_daemon_dbus_setup (loop, path);
+#endif
 	
 	g_main_loop_run (loop);
+
+#ifdef WITH_DBUS
+	gnome_keyring_daemon_dbus_cleanup ();
+#endif
 	
 	cleanup_socket_dir ();
 	return 0;
 }
+
